/**
 * @file bucket.h
 * Header file for Bucket class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _BUCKET_H
#define _BUCKET_H

#include "main.h"
#include "object.h"

#include <string>
using namespace std;

#include <ode/ode.h>

/**
 * Buckets consist of 5 rectangles (4 sides and a bottom) with an
 * open top.  
 *
 * Useful as a goal-related object (get the ball into the bucket).
 */
class Bucket : public Object {
public:

    static string get_type_str_static( void ) { return "bucket"; }

    static Object* create( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element )
        { return new Bucket( world_id, space_id, element ); }

    virtual int load( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element, bool load_parent = true );

    /** 
     * Bucket constructor. 
     *
     * @param world_id ode world in which to place the bucket's body
     * @param space_id ode space in which to place the bucket's geometry
     * @param element  xml element from the level file from which the 
     *                 necessary parameters are taken to construct the bucket
     */
    Bucket( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element );

    virtual ~Bucket( void );

    /**
     * Render the transformed bucket's model to the screen.  
     * 
     * Assumes a correct opengl state is in place.
     *
     * @param selected unused
     * 
     * @see Object::Render
     */
    virtual void render( bool selected = false );

    virtual string get_type_str( void ) const { return get_type_str_static(); }

private:

    dGeomID _geom_ids[ 5 ]; ///< 5 rectangle geometries of the bucket
};

#endif // header guard
