/**
 * @file connectable.h
 * Header for Connectable class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _CONNECTABLE_H
#define _CONNECTABLE_H

#include "main.h"

#include <ClanLib/core.h>

#include <ode/ode.h>

class Object;

/**
 * Connects one object to another object.
 *
 * The connected object's status can be changed based on the status of the
 * connector object, i.e. turn the connected object on/off when the connector
 * object is collided with.
 */
class Connectable {
public:

    Connectable( void ) : _connection( NULL ) { }

    virtual int load( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element, bool load_parent = true );

    //@{
    /** @name Accessors
     * 
     * Accessor functions
     */
    void set_connection( Object* object ) { _connection = object; }
    Object* get_connection() { return _connection; }
    const Object* get_connection() const { return _connection; }
    //@}

private:

    Object* _connection;    ///< connected object whose status is modifiable
};

#endif // header guard
