/**
 * @file conveyorbelt.cpp
 * Implementation of ConveyorBelt class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */

#include "conveyorbelt.h"
#include "model.h"

#include <gfx/gl.h>

// add a loader function for this object
static RegisterLoader dummy( ConveyorBelt::get_type_str_static(), &ConveyorBelt::create );

int ConveyorBelt::load( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element, bool load_parent )
{
    if ( load_parent ) {
        ShapeObject::load( world_id, space_id, element );
    }

    _texture_slide = 0.0;

    return 0;
}

void ConveyorBelt::tick( double t )
{
    if ( is_on() ) {
        _texture_slide = fmod( _texture_slide + t , 1.0 );
    }
}

void ConveyorBelt::render( bool selected )
{
    glMatrixMode( GL_TEXTURE );
    glPushMatrix();

    glTranslated( 0.0, _texture_slide, 0.0 );

    glMatrixMode( GL_MODELVIEW );

    Object::render();

    glMatrixMode( GL_TEXTURE );
    glPopMatrix();

    glMatrixMode( GL_MODELVIEW );
}

void ConveyorBelt::collide( dContact* contact )
{
    if( !contact ) {
        cerr << "ConveyorBelt::collide called with NULL contact!" << endl;
        return;
    }

    Object::collide( contact );

    if ( !is_on() ) {
        // only push objects if we're on.
        return;
    }

    // find the tangent in the direction of motion
    Mat4 rot;
    dMatrix3ToMat4( rot, dBodyGetRotation( _body_id ) );

    Vec4r fdir1_4 = Vec4r( get_forward_vec(), 0 );
    Vec3r fdir1 = vec_dropw( fdir1_4 );
    unitize( fdir1 );

    // set the surface in motion
    contact->surface.mode |= dContactFDir1 | dContactMotion1;
    contact->surface.motion1 = 75.0f;

    Vec3TodVector3( contact->fdir1, fdir1 );
}

