/**
 * @file conveyorbelt.h
 * Header file for ConveyorBelt class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _CONVEYORBELT_H
#define _CONVEYORBELT_H

#include "shapeobject.h"

#include <string>
using namespace std;

#include <ode/ode.h>

/**
 * Transporting conveyor belt.
 * 
 * When the conveyor belt is turned on it will transport other objects
 * which land on it from one of its ends to the other.  The direction of 
 * transport is defined as the conveyor belt's forward vector.
 */
class ConveyorBelt : public ShapeObject {
public:

    static string get_type_str_static( ) { return "conveyorbelt"; }

    static Object* create( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element )
        { return new ConveyorBelt( world_id,  space_id, element ); }

    virtual int load( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element, bool load_parent = true );

    ConveyorBelt( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element ) : 
        ShapeObject( world_id, space_id, element )
    { load( world_id, space_id, element, false ); }

    /**
     * Advance the conveyor belt's state.  
     * 
     * Currently amounts to updating the belt's animation.
     *
     * @param t amount of time (in seconds) to advance the state of the
     *          conveyor belt.
     */    
    virtual void tick( double t );

    /**
     * Render the transformed belt's model to the screen.  
     * 
     * Assumes a correct opengl state is in place.
     *
     * @param selected unused
     * 
     * @see Object::Render
     */
    virtual void render( bool selected = true );

    /**
     * When another object collides with the belt, set the contact joint's
     * properties appropriately.
     *
     * @param contact ode dContact* which determines the properties for the
                      contact joint generated by the current collision
     */
    virtual void collide( dContact* contact );

    virtual string get_type_str( void ) const { return get_type_str_static( ); }

private:

    double _texture_slide;  ///< amount by which to animate the belt's texture each render loop
};

#endif // header guard
