/**
 * @file seesaw.h
 * Header for Seesaw class
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _SEESAW_H
#define _SEESAW_H

#include "main.h"
#include "shapeobject.h"

#include <string>
using namespace std;

#include <gfx/gl.h>

#include <ode/ode.h>

/**
 * A seesaw made up from a long board and a triangle-ish fulcrum.
 */
class Seesaw : public ShapeObject {
public:

    static string get_type_str_static( void ) { return "seesaw"; }

    static Object* create( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element )
        { return new Seesaw( world_id, space_id, element ); }

    virtual int load( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element, bool load_parent = true );

    Seesaw( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element );
    virtual ~Seesaw( void );

    /**
     * Render the seesaw's model to the screen.
     *
     * Assumes a correct opengl state is in place.
     *
     * @param selected unused
     */
    virtual void render( bool selected = false );

    /**
     * Notify the object that the simulation has started.  
     *
     * It will need to save its current state so that it can be reset when the 
     * simulation stops.
     */
    virtual void start_sim( void );

    virtual string get_type_str( ) const { return get_type_str_static( ); }

private:

    dJointID _hinge;        ///< the hinge joint between the board and fulcrum
};

#endif // header guard
