/**
 * @file actors/shapeobject.cpp
 * Implementation of ShapeObject class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#include "shapeobject.h"
#include "model.h"
#include "shape.h"

// add a loader function for this object
static RegisterLoader dummy( ShapeObject::get_type_str_static(), &ShapeObject::create );

int ShapeObject::load( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element, bool load_parent )
{
    if ( load_parent ) {
        Object::load( world_id, space_id, element );
    }

    _shape = get_shape( element, "shape" );

    Vec3 size = _model->get_max() - _model->get_min();
    size = vec_mult( _scale, size );

    if ( _geom_id ) {
        dGeomDestroy( _geom_id );
    }

    // create ODE shape
    switch ( _shape ) {
    case SHAPE_BOX:
        dMassSetBoxTotal( &_dmass, _mass, size[ 0 ], size[ 1 ], size[ 2 ] );
        _geom_id = dCreateBox( space_id, size[ 0 ], size[ 1 ], size[ 2 ] );
        break;

    case SHAPE_SPHERE:
        {
            double radius = min( size[ 0 ], min( size[ 1 ], size[ 2 ] ) ) / 2.0;

            dMassSetSphereTotal( &_dmass, _mass, radius );
            _geom_id = dCreateSphere( space_id, radius );
            break;
        }

    case SHAPE_CYLINDER:
        {
            double radius = min( size[ 0 ], min( size[ 1 ], size[ 2 ] ) ) / 2.0;
            double length = max( size[ 0 ], max( size[ 1 ], size[ 2 ] ) ) - 2.0 * radius;

            dMassSetCappedCylinderTotal( &_dmass, _mass, 3, radius, length );
            _geom_id = dCreateCCylinder( space_id, radius, length );
            break;
        }
    }

    _setup_geom();

    return 0;
}
