/**
 * @file shapeobject.h
 * Header for ShapeObject class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _SHAPEOBJECT_H
#define _SHAPEOBJECT_H

#include "main.h"
#include "object.h"
#include "utils.h"

#include <string>
using namespace std;

/**
 * A ShapeObject is made up of a primitive shape, both in the renderede model 
 * and ode geometry
 *
 */
class ShapeObject : public Object {
public:

    static string get_type_str_static( ) { return "shape"; }
    static Object* create( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element )
        { return new ShapeObject( world_id, space_id, element ); }

    virtual int load( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element, bool load_parent = true );

    ShapeObject( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element ) : 
        Object( world_id, space_id, element )
    { load( world_id, space_id, element, false ); }

    virtual string get_type_str( ) const { return get_type_str_static(); }
    virtual string get_info( ) const
        { return "shape=\"" + to_string( _shape ) + "\" " + Object::get_info(); }

private:    

    Shape _shape;   ///< primitive shape this object represents
};

#endif // header guard
