/**
 * @file trigger.cpp
 * Implementation for Trigger class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */

#include "trigger.h"
#include "model.h"

// add a loader function for this object
static RegisterLoader dummy( Trigger::get_type_str_static(), &Trigger::create );

int Trigger::load( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element, bool load_parent )
{
    if ( load_parent ) {
        ShapeObject::load( world_id, space_id, element );
    }

    Connectable::load( world_id, space_id, element );

    _timer = 0.0;
    // FIXME: might want to be able to set the on/off materials in the
    // level file.
    _model->set_material( _on ? green : red );

    return 0;
}

void Trigger::stop_sim( void )
{
    Object::stop_sim();

    _model->set_material( _on ? green : red );
}

void Trigger::tick( double t )
{
    if ( _timer > 0.0 ) {
        _timer -= t;
    }

    if ( _timer <= 0.0 ) {
        _model->set_material( red );
        set_on( false );
        get_connection()->set_on( false );
    }
}

void Trigger::collide( dContact* contact )
{
    Object::collide( contact );

    _model->set_material( green );
    set_on( true );
    get_connection()->set_on( true );

    _timer = 5.0;
}

string Trigger::get_info( ) const
{
    return "connectionID=\"" + to_string( get_connection()->get_id() ) + "\" " +
        ShapeObject::get_info();
}

