/**
 * @file actors/trigger.h
 * Header for Trigger class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _TRIGGER_H
#define _TRIGGER_H

#include "main.h"
#include "shapeobject.h"
#include "connectable.h"

#include <string>
using namespace std;

#include <ode/ode.h>

/**
 * Triggers represent the on/off switch of some other object.
 *
 * When the trigger is collided with it will toggle the activation status of 
 * the object it is connected to.
 */
class Trigger : public ShapeObject, public Connectable {
public:

    static string get_type_str_static( ) { return "trigger"; }

    static Object* create( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element )
        { return new Trigger( world_id, space_id, element ); }
    
    virtual int load( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element, bool load_parent = true );

    Trigger( dWorldID world_id, dSpaceID space_id, const CL_DomElement& element ) : 
        ShapeObject( world_id, space_id, element )
    { load( world_id, space_id, element, false ); }

    /**
     * Notify the object that the simulation has stopped.  
     *
     * It wil need to reset its state to what it was before the simulation 
     * started.
     */
    virtual void stop_sim( void );

    /**
     * Advance the trigger's state.
     *
     * @param t amount of time (in seconds) to advance the trigger's state
     */
    virtual void tick( double t );

    /**
     * Notify the object that it has been collided with.
     *
     * Allows the object to set salient properties of the contact joint.
     *
     * @param contact ode dContact* which determines the properties for the
                      contact joint generated by the current collision
     */
    virtual void collide( dContact* contact );

    virtual string get_type_str( void ) const { return get_type_str_static(); }
    virtual string get_info( void ) const;

private:

    double _timer;      ///< how long to stay on
};

#endif // header guard
