/**
 * @file worldbox.h
 * Header file for Worldbox class.
 *
 * Copyright 2003-2004 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _WORLDBOX_H
#define _WORLDBOX_H

#include "main.h"
#include "gfxutils.h"

#include <string>
using namespace std;

#include <ode/ode.h>

class Material;

/**
 * A boundary box which contains a level along with a level background.
 */
class WorldBox {
public:

    /**
     * WorldBox constructor.
     *
     * @param space_Id  ode spaceID in which to place the worldbox's geometry
     * @param size      total size of the box in each dimension, centered on the
     *                  orign
     * @param box_mat   Material of the walls of the box
     * @param cloud_mat Material of the background behind the box
     */
    WorldBox( dSpaceID space_id, Vec3 size, const Material& wall_mat, const Material& background_mat );

    ~WorldBox( void );

    /**
     * Render the worldbox.
     *
     * Don't draw the walls blocking the camera's view into the world
     *
     * @param camera_position current position of the camera
     */
    void render( Vec3r camera_position );

    //@{
    /** @name Accessors
     *
     * Accessor functions.
     */
    Vec3r get_size( void ) { return _size; }

    string WorldBox::get_info( void ) const;
    //@}

private:

    Material* _wall;           ///< Material of the box's walls
    Material* _background;      ///< Material of the background

    dGeomID _geom_id[ 6 ];      ///< geom_id's of the 6 walls

    Vec3r _size;                ///< size of the worldbox along each dimension

    Vec3r _min, _max;           ///< extents of the worldbox

    double _background_ds;      ///< amount to move the background's s texture coord
    double _background_dt;      ///< amount to mave the background's t texture coord
};

#endif // header guard
