/**
 * @file effect.cc
 * Implementation of Effect class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#include "effects.h"
#include "object.h"

/**
 * NOTE: This *MUST* be NULL before ANY GLOBAL CONTRUCTORS.  I think static
 * global variables are automatically 0, so we should be okay.
 * RegisterEffect depends on this variable being NULL before its global
 * constructors are called.  This is actually an issue: if RegisterEffect's
 * constructor were called before this was initialized to NULL, baaaad
 * things would happen.
 * @see RegisterEffect
 */
static list<Effect*> *effects_list = NULL;

RegisterEffect::RegisterEffect(Effect *effect)
{
    if ( effects_list == NULL ) {
        effects_list = new list<Effect*>();
    }
    effects_list->push_back( effect );
}

RegisterEffect::iterator RegisterEffect::begin()
{
    assert( effects_list );

    return effects_list->begin();
}

RegisterEffect::iterator RegisterEffect::end()
{
    assert( effects_list );

    return effects_list->end();
}

void Effect::pre_apply( Object *who )
{
}

void Effect::apply_to( Object *who )
{
    EffectHandler func = EffectHandler( find_handler( who ) );
    if ( ! func ) {
        return;
    }
    this->pre_apply( who );
    (who->*func)( this );
}

void Effect::apply_to_all( void )
{
    RecList& reclist = get_receiver_list();
    RecList::iterator it;
    Object *who;
    EffectHandler func;

    for ( it = reclist.begin(); it != reclist.end(); it++ ) {
        who = (*it).first;
        func = ( EffectHandler )(*it).second;
        this->pre_apply( who );
        (who->*func)( this );
    }
}

void Effect::add_handler( Object *who )
{
    EffectHandler func = who->get_effect_handler( this );
    if (func) {
        (get_receiver_list())[who] = func;
    }
}

void Effect::clear_handlers( void )
{
    get_receiver_list().clear();
}

const Vec3r Effect::get_origin( void )
{
    Object *obj = dynamic_cast<Object*>( _sender );

    assert( obj );

    return obj->get_position();
}
