/**
 * @file shapemodel.cpp
 * Implementation of ShapeModel class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#include "shapemodel.h"

#include <gfx/gl.h>

ShapeModel::ShapeModel( Shape shape, Vec3r size ) :
    _shape( shape ), _size( size )
{
    _gen_object();
}

ShapeModel::ShapeModel( Shape shape, Vec3r size, const Material& mat ) :
    Model( mat ), _shape( shape ), _size( size )
{
    _gen_object();
}

ShapeModel::~ShapeModel()
{
    if ( _quadric ) {
        gluDeleteQuadric( _quadric );
    }
}

void ShapeModel::render()
{
    Model::render();

    switch ( _shape ) {
    case SHAPE_SPHERE:
        gluSphere( _quadric, _max[ 0 ], 20, 10 );
        break;

    case SHAPE_CYLINDER:
        draw_capped_cylinder( _max[ 2 ], _max[ 0 ] );
        break;

    case SHAPE_BOX:
        draw_solid_cube( _min, _max );
        break;
    }
}

void ShapeModel::_gen_object()
{
    _min = -_size / 2.0;
    _max = _size / 2.0;

    switch ( _shape ) {
    case SHAPE_SPHERE:
        _quadric = gluNewQuadric();
        gluQuadricTexture( _quadric, GL_TRUE );
        break;

    case SHAPE_CYLINDER:
        _min[ 2 ] *= 2.0;
        _max[ 2 ] *= 2.0;

    case SHAPE_BOX:
        _quadric = NULL;
        break;
    }
}
