/**
 * @file shapemodel.h
 * Header for ShapeModel class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _SHAPEMODEL_H
#define _SHAPEMODEL_H

#include "main.h"
#include "model.h"
#include "gfxutils.h"

#include <gfx/gl.h>

class Material;

/**
 * A model for basic shapes.
 */
class ShapeModel : public Model {
public:

    /**
     * ShapeModel constructor.
     *
     * @param shape the shape primitive to use in rendering
     * @param size  the size of the shape
     */
    ShapeModel( Shape shape, Vec3r size );

    /**
     * ShapeModel constructor.
     *
     * @param shape the shape primitive to use in rendering
     * @param size  the size of the shape
     * @param mat   the material of the model
     */
    ShapeModel( Shape shape, Vec3r size, const Material& mat );

    virtual ~ShapeModel( void );

    /**
     * Render the model to the screen.
     */
    virtual void render( void );

    //@{
    /** @name Accessors
     *
     * Accessor functions.
     */
    virtual Vec3r get_min( void ) { return _min; }
    virtual Vec3r get_max( void ) { return _max; }
    //@}

private:

    /**
     * Generate the quadric.
     */
    void _gen_object( void );

    GLUquadric *_quadric;       ///< glu quadric

    Shape _shape;               ///< type of shape

    Vec3r _size;                 ///< size of shape
    Vec3r _min, _max;            ///< extents of shape
};

#endif // header guard
