/**
 * @file gfxutils.h
 * Useful libgfx utils and defines.
 *
 * Copyright 2003-2004 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _GFXUTILS_H
#define _GFXUTILS_H

#include <gfx/vec4.h>
#include <gfx/mat4.h>

#include <ode/ode.h>

typedef TVec3<dReal> Vec3r;
typedef TVec4<dReal> Vec4r;

typedef Vec3f Color;

template <class T>
inline dReal operator*(const TVec4<T>& u, const Vec4r& v)
    { return dReal(u[0]*v[0] + u[1]*v[1] + u[2]*v[2] + u[3]*v[3] ); }

//template <class T>
inline dReal operator*(const Vec3r& u, const Vec3r& v)
    { return dReal(u[0]*v[0] + u[1]*v[1] + u[2]*v[2]); }

inline Vec4r operator*(const Mat4& m, const Vec4r& v)
    { return Vec4r(m[0]*v, m[1]*v, m[2]*v, m[3]*v); }

//
// Transform a homogeneous 3-vector and reproject into normal 3-space
//
inline Vec3r operator*(const Mat4& m, const Vec3r& v)
{
    Vec4r u=Vec4r(v,1);
    double w=m[3]*u;

    if(w==0.0)  return Vec3r(m[0]*u, m[1]*u, m[2]*u);
    else        return Vec3r(m[0]*u/w, m[1]*u/w, m[2]*u/w);
}

#endif // header guard
