/**
 * @file odeutils.h
 * Header for ode-related utility functions.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _ODEUTILS_H
#define _ODEUTILS_H

#include "main.h"
#include "gfxutils.h"

#include <iostream>
using namespace std;

#include <gfx/mat4.h>
#include <gfx/quat.h>

#include <ode/ode.h>

//@{
/** @name ode utility functions
 *
 * ode-related utility functions.
 *
 * Since ODE likes to return dReal *'s and since C++ doesn't like to return
 * arrays we have all these nice conversion functions, the first argument is
 * always the target and the second argument is always the source
 *
 * NOTE: ODE stores quaternions with scalar first, then vector
 * NOTE: ODE stores matrices in row-major order (dmat[4*i + j] = row i,
 * col j)
 */

void Mat4TodMatrix3( dMatrix3 dmat3, const Mat4 mat4 );

void dMatrix3ToMat4( Mat4& mat4, const dMatrix3 dmat3 );
void dMatrix3TodMatrix3( dMatrix3 dmat3_1, const dMatrix3 dmat3_2 );

/**
 * Useful for pushing an object's rotation onto the modelview_matrix stack.
 */
void dMatrix3ToOpenGL( dReal oglMat[ 16 ], const dMatrix3 dMat3 );

void dRealPtrTodMatrix4( dMatrix4 dmat4, const dReal* ptr );
void dRealPtrTodMatrix3( dMatrix3 dmat3, const dReal* ptr );

inline void dRealPtrToVec3( Vec3r& v, const dReal* ptr )
    { v = Vec3r( ptr ); }

inline void Vec3TodVector3( dVector3 dv, const Vec3r &v )
    { dv[0] = v[0]; dv[1] = v[1]; dv[2] = v[2]; }

inline void dRealPtrTodQuaternion( dQuaternion dq, const dReal* q )
    { dq[0] = q[0]; dq[1] = q[1]; dq[2] = q[2]; dq[3] = q[3]; }

inline void dQuaternion_to_quat( Quat& q, const dQuaternion dq )
    { q = Quat( dq[1], dq[2], dq[3], dq[0] ); }

inline void quat_to_dQuaternion( dQuaternion dq, const Quat& q )
    { dq[1] = q.vector()[0]; dq[2] = q.vector()[1]; dq[3] = q.vector()[2]; dq[0] = q.scalar(); }

//@}

#endif // header guard
